from torch_geometric.nn import GCNConv
from torch_geometric.nn.pool.topk_pool import topk,filter_adj
from torch.nn import Parameter
import torch


class SAGPool(torch.nn.Module):
    def __init__(self,in_channels,ratio=0.8,Conv=GCNConv,non_linearity=torch.tanh, cus_drop_ratio= 0.5, multi_channel=1):
        super(SAGPool,self).__init__()
        self.in_channels = in_channels
        self.ratio = ratio
        self.score_layer = Conv(in_channels, multi_channel)
        self.non_linearity = non_linearity
        self.dropout = torch.nn.Dropout(cus_drop_ratio)
        self.multi_channel =multi_channel
    def forward(self, x, edge_index, edge_attr=None, batch=None):
        if batch is None:
            batch = edge_index.new_zeros(x.size(0))
            #x = x.unsqueeze(-1) if x.dim() == 1 else x
        score = self.score_layer(x,edge_index)

        sc = self.dropout(score.abs().sum(-1).view(-1,1))

        perm = topk(sc.view(-1), self.ratio, batch)


        x_list = []
        for i in range(self.multi_channel):
            x_list.append(x[perm] * self.non_linearity(score[perm,i]).view(-1, 1))

        x = torch.cat(tuple(x_list),dim=1)


        batch = batch[perm]
        edge_index, edge_attr = filter_adj(
                edge_index, edge_attr, perm, num_nodes=score.size(0))


        return x, edge_index, edge_attr, batch, perm